@@ -4,7 +4,7 @@
  * GChemPaint library
  * document.cc 
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,12 +28,17 @@
 #include "document.h"
 #include "settings.h"
 #include "docprop.h"
+#include "fragment.h"
 #include "reaction.h"
 #include "mesomery.h"
+#include "molecule.h"
+#include "residue.h"
 #include "text.h"
 #include "theme.h"
 #include "tool.h"
 #include "window.h"
+#include <gcu/objprops.h>
+#include <gcu/loader.h>
 #include <goffice/math/go-rangefunc.h>
 #include <unistd.h>
 #include <libgen.h>
@@ -399,9 +404,59 @@ void Document::ExportOB ()
 	}
 }
 
-void Document::Print (GnomePrintContext *pc, gdouble width, gdouble height)
+void Document::DoPrint (GtkPrintOperation *print, GtkPrintContext *context)
 {
-	m_pView->Print (pc, width, height);
+	cairo_t *cr;
+	double width, height, x, y, w, h;
+
+	// TODO: support multiple pages
+	cr = gtk_print_context_get_cairo_context (context);
+	width = gtk_print_context_get_width (context);
+	height = gtk_print_context_get_height (context);
+	// adjust position and size
+	GtkWidget* widget = m_pView->GetWidget ();
+	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (widget), "data");
+	ArtDRect rect;
+	pData->GetObjectBounds (this, &rect);
+	double scale = 1.;
+	x = rect.x0;
+	y = rect.y0;
+	w = rect.x1 - rect.x0;
+	h = rect.y1 - rect.y0;
+	cairo_save (cr);
+	cairo_rectangle (cr, 0, 0, width, height);
+	cairo_clip (cr);
+	switch (GetScaleType ()) {
+	case GCU_PRINT_SCALE_NONE:
+		break;
+	case GCU_PRINT_SCALE_FIXED:
+		scale *= Printable::GetScale ();
+		x *= scale;
+		y *= scale;
+		w *= scale;
+		h *= scale;
+		break;
+	case GCU_PRINT_SCALE_AUTO:
+		if (GetHorizFit () && GetVertFit ())
+			scale *= min (width / w, height / h);
+		else if (GetHorizFit ())
+			scale *= width / w;
+		else if (GetVertFit ())
+			scale *= height / h;
+		x *= scale;
+		y *= scale;
+		w *= scale;
+		h *= scale;
+		break;
+	}
+	if (GetHorizCentered ())
+		x -= (width - w) / 2;
+	if (GetVertCentered ())
+		y -= (height - h) / 2;
+	cairo_translate (cr, -x, -y);
+	cairo_scale (cr, scale, scale);
+	m_pView->Render (cr);
+	cairo_restore (cr);
 }
 
 void Document::AddAtom (Atom* pAtom)
@@ -525,10 +580,13 @@ static int cb_xml_to_vfs (GnomeVFSHandle *handle, const char* buf, int nb)
 
 void Document::Save ()
 {
+	if (m_bReadOnly)
+		SetReadOnly (false);
 	if (!m_filename || !m_bWriteable || m_bReadOnly)
 		return;
 	xmlDocPtr xml = NULL;
 	char *old_num_locale, *old_time_locale;
+	m_SavedResidues.clear ();
 	
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
@@ -541,9 +599,11 @@ void Document::Save ()
 		else {
 			xml = BuildXMLTree ();
 			xmlSetDocCompressMode (xml, CompressionLevel);
-			
-			xmlIndentTreeOutput = true;
-			xmlKeepBlanksDefault (0);
+
+			if (!CompressionLevel) {
+				xmlIndentTreeOutput = true;
+				xmlKeepBlanksDefault (0);
+			}
 		
 			GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
 			gnome_vfs_get_file_info (m_filename, info, GNOME_VFS_FILE_INFO_DEFAULT);
@@ -583,6 +643,7 @@ void Document::Save ()
 	g_free (old_num_locale);
 	setlocale (LC_TIME, old_time_locale);
 	g_free (old_time_locale);
+	m_SavedResidues.clear ();
 }
 
 bool Document::Load (xmlNodePtr root)
@@ -1189,7 +1250,7 @@ void Document::ExportImage (string const &filename, const char* type, int resolu
 void Document::SetReadOnly (bool ro)
 {
 	m_bReadOnly = ro;
-	if (!ro && (m_FileType != "application/x-gchempaint")) {
+	if (!ro && (m_FileType != "application/x-gchempaint") && !Loader::GetSaver (m_FileType.c_str ())) {
 		OBFormat *f = OBConversion::FormatFromMIME (m_FileType.c_str ());
 		m_bReadOnly = (f == NULL)? true: f->Flags () & NOTWRITABLE;
 	}
@@ -1324,4 +1385,84 @@ double Document::GetMedianBondLength ()
 	return result;
 }
 
+bool Document::SetProperty (unsigned property, char const *value)
+{
+	switch (property) {
+	case GCU_PROP_DOC_FILENAME:
+		SetFileName (value, m_FileType.c_str ());
+		break;
+	case GCU_PROP_DOC_MIMETYPE:
+		m_FileType = value;
+		break;
+	case GCU_PROP_DOC_TITLE:
+		SetTitle (value);
+		if (m_Window)
+			m_Window->SetTitle (GetTitle ());
+		break;
+	case GCU_PROP_DOC_COMMENT:
+		g_free (m_comment);
+		m_comment = g_strdup (value);
+		break;
+	case GCU_PROP_DOC_CREATOR:
+		g_free (m_author);
+		m_author = g_strdup (value);
+		break;
+	case GCU_PROP_DOC_CREATION_TIME:
+		g_date_set_parse (&CreationDate, value);
+		break;
+	case GCU_PROP_DOC_MODIFICATION_TIME:
+		g_date_set_parse (&RevisionDate, value);
+		break;
+	case GCU_PROP_THEME_BOND_LENGTH: {
+		char *end;
+		double length = strtod (value, &end);
+		if (*end != 0)
+			return false;
+		gcu::Document::m_Scale = m_Theme->GetBondLength () / length; 
+	}
+	}
+	return true;
+}
+
+GtkWindow *Document::GetGtkWindow ()
+{
+	return (m_Window)? m_Window->GetWindow (): NULL;
+}
+
+void Document::SaveResidue (Residue const *r, xmlNodePtr node)
+{
+	if (m_SavedResidues.find (r) == m_SavedResidues.end ()) {
+		m_SavedResidues.insert (r);
+		xmlNewProp (node, (xmlChar const *) "raw", (xmlChar const *) const_cast <Molecule*> (r->GetMolecule ())->GetRawFormula ().c_str ());
+		xmlNewProp (node, (xmlChar const *) "generic", (xmlChar const *) (r->GetGeneric ()? "true": "false"));
+		map<string, bool> const &symbols = r->GetSymbols ();
+		map<string, bool>::const_iterator i = symbols.begin (), iend = symbols.end ();
+		string s = (*i).first;
+		for (i++; i != iend; i++) {
+			s += ";";
+			s += (*i).first;
+		}
+		xmlNodePtr child = xmlNewDocNode (node->doc, NULL, (xmlChar const *) "symbols", (xmlChar const *) s.c_str ());
+		xmlAddChild (node, child);
+		map<string, string> const &names = r->GetNames ();
+		map<string, string>::const_iterator j, jend = names.end ();
+		j = names.find ("C");
+		if (j != jend) {
+			child = xmlNewDocNode (node->doc, NULL, (xmlChar const *) "name", (xmlChar const *) (*j).second.c_str ());
+			xmlAddChild (node, child);
+		}
+		for (j = names.begin (); j != jend; j++) {
+			if ((*j).first == "C")
+				continue;
+			child = xmlNewDocNode (node->doc, NULL, (xmlChar const *) "name", (xmlChar const *) (*j).second.c_str ());
+			xmlNodeSetLang (child, (xmlChar const *) (*j).first.c_str ());
+			xmlAddChild (node, child);
+		}
+		child = const_cast <Molecule*> (r->GetMolecule ())->Save (node->doc);
+		if (child) {
+			xmlAddChild (node, child);
+		}
+	}
+}
+
 }	//	namespace gcp
