@@ -24,6 +24,12 @@
 
 #include "config.h"
 #include "application.h"
+#include "cmd-context.h"
+#include "loader.h"
+#include <goffice/goffice.h>
+#include <goffice/app/io-context.h>
+#include <gsf-gnome/gsf-input-gnomevfs.h>
+#include <gsf-gnome/gsf-output-gnomevfs.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glade/glade.h>
@@ -34,11 +40,32 @@
 #include <sys/stat.h>
 #include <cmath>
 #include <cstdlib>
+#include <cstring>
+#include <clocale>
+#include <set>
 
-using namespace gcu;
+using namespace std;
+
+namespace gcu
+{
+
+#ifdef HAVE_GO_CONF_SYNC
+GOConfNode *Application::m_ConfDir = NULL;
+#endif
+
+static set<Application *> Apps;
 
 Application::Application (string name, string datadir, char const *help_name, char const *icon_name)
 {
+#ifdef HAVE_GO_CONF_SYNC
+	if (m_ConfDir == NULL) {
+		libgoffice_init ("gchemutils");
+		m_ConfDir = go_conf_get_node (NULL, GCU_CONF_DIR);
+	}
+#else
+	libgoffice_init ();
+#endif
+	Apps.insert (this);
 	static bool first_call = true;
 	Name = name;
 	char const *szlang = getenv ("LANG");
@@ -57,8 +84,13 @@ Application::Application (string name, string datadir, char const *help_name, ch
 		GConfEntry* entry = gconf_client_get_entry (cli, "/desktop/gnome/applications/help_viewer/exec", NULL, true, NULL);
 		if (entry) {
 			value = gconf_value_get_string (gconf_entry_get_value (entry));
-			if (value) HelpBrowser = value;
-		}
+			if (value) {
+				HelpBrowser = value;
+				if (HelpBrowser.find ("nautilus") != string::npos)
+					HelpBrowser = "yelp";
+			}
+		} else
+			HelpBrowser = "yelp";
 		entry = gconf_client_get_entry (cli, "/desktop/gnome/applications/browser/exec", NULL, true, NULL);
 		if (entry) {
 			value = gconf_value_get_string (gconf_entry_get_value (entry));
@@ -106,9 +138,15 @@ Application::Application (string name, string datadir, char const *help_name, ch
 
 Application::~Application ()
 {
-	if (CurDir)
-		g_free (CurDir);
 	g_object_unref (m_RecentManager);
+	Apps.erase (this);
+	if (Apps.empty ()) {
+#ifdef HAVE_GO_CONF_SYNC
+		go_conf_free_node (m_ConfDir);
+		m_ConfDir = NULL;
+#endif
+		libgoffice_shutdown ();
+	}
 }
 
 void Application::OnHelp (string tag)
@@ -137,9 +175,12 @@ bool Application::HasHelp ()
 
 void Application::SetCurDir (char const* dir)
 {
-	if (CurDir)
-		g_free (CurDir);
-	CurDir = g_strdup (dir);
+	CurDir = dir;
+}
+
+void Application::SetCurDir (string const &dir)
+{
+	CurDir = dir;
 }
 
 void Application::OnMail (char const *MailAddress)
@@ -231,3 +272,62 @@ char const *Application::GetPixbufTypeName (string& filename, char const *mime_t
 	return gdk_pixbuf_format_get_name (format);
 }
 
+void Application::RemoveDocument (Document *Doc)
+{
+	m_Docs.erase (Doc);
+	if (m_Docs.size () == 0 && gtk_main_level ())
+		NoMoreDocsEvent ();
+}
+
+bool Application::Load (std::string const &uri, const gchar *mime_type, Document* Doc)
+{
+	Loader *l = Loader::GetLoader (mime_type);
+	if (!l)
+		return false;
+	string old_num_locale = setlocale (LC_NUMERIC, NULL);
+	setlocale(LC_NUMERIC, "C");
+	GError *error = NULL;
+	GsfInput *input = gsf_input_gnomevfs_new (uri.c_str (), &error);
+	if (error) {
+		g_error_free (error);
+	}
+	IOContext *io = gnumeric_io_context_new (gcu_get_cmd_context ());
+	bool ret = l->Read (Doc, input, mime_type, io);
+	g_object_unref (input);
+	g_object_unref (io);
+	setlocale (LC_NUMERIC, old_num_locale.c_str ());
+	return ret;
+}
+
+bool Application::Save (std::string const &uri, const gchar *mime_type, Document* Doc)
+{
+	Loader *l = Loader::GetSaver (mime_type);
+	if (!l)
+		return false;
+	string old_num_locale = setlocale (LC_NUMERIC, NULL);
+	setlocale(LC_NUMERIC, "C");
+	GError *error = NULL;
+	GsfOutput *output = gsf_output_gnomevfs_new (uri.c_str (), &error);
+	if (error) {
+		g_error_free (error);
+	}
+	IOContext*io  = gnumeric_io_context_new (gcu_get_cmd_context ());
+	bool ret = l->Write (Doc, output, mime_type, io);
+	g_object_unref (output);
+	g_object_unref (io);
+	setlocale (LC_NUMERIC, old_num_locale.c_str ());
+	return ret;
+}
+
+#ifdef HAVE_GO_CONF_SYNC
+GOConfNode *Application::GetConfDir ()
+{
+	if (m_ConfDir == NULL) {
+		libgoffice_init ("gchemutils");
+		m_ConfDir = go_conf_get_node (NULL, GCU_CONF_DIR);
+	}
+	return m_ConfDir;
+}
+#endif
+
+}	//	namespace gcu
