@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * object.h 
  *
- * Copyright (C) 2002-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -25,6 +25,7 @@
 #ifndef GCU_OBJECT_H
 #define GCU_OBJECT_H
 
+#include "macros.h"
 #include "matrix2d.h"
 #include <glib.h>
 #include <libxml/parser.h>
@@ -34,12 +35,9 @@
 #include <string>
 #include <stdexcept>
 #include <gtk/gtk.h>
-#include <libgnomeprint/gnome-print.h>
 
 #define square(x) ((x)*(x))
 
-using namespace std;
-
 namespace gcu
 {
 
@@ -169,9 +167,10 @@ the Object::AddType method.
 	Each Object instance maintains a list of its children. If object has already a parent, it will be removed from its
 	parent children list. The new parent Object must have a Document ancestor to ensure that Ids are unique.
 */
-	void AddChild (Object* object);
+	virtual void AddChild (Object* object);
 /*!
-	Used to get the Molecule in the Object instances ancestors. 
+	Used to get the Molecule in the Object instances ancestors.  Overloaded methods should call the
+	base class Object::AddChild.
 	
 	@return the first Object of type MoleculeType encountered when exploring
 	the Objects tree or NULL if none is found.
@@ -222,14 +221,14 @@ To search the Object in lower shells of the tree, use the Object::GetDescendant
 Use this function to retrieve the first child of the object and initialize the iterator.
 @return the first child of the object or NULL.
 */
-	Object* GetFirstChild (map<string, Object*>::iterator& i);
+	Object* GetFirstChild (std::map<std::string, Object*>::iterator& i);
 /*!
 @param i: a C++ std::map iterator initialized by Object::GetFirstChild.
 
 Use this method to iterate through the list of the Object children.
 @return the next child of the object or NULL.
 */
-	Object* GetNextChild (map<string, Object*>::iterator& i);
+	Object* GetNextChild (std::map<std::string, Object*>::iterator& i);
 /*!
 @param Id: the Id of the descendant searched.
 
@@ -357,12 +356,6 @@ unless the application uses another mechanism.
 */
 	virtual void Add (GtkWidget* w);
 /*!
-@param pc: the GnomePrintContext to which the document is printed.
-
-This method might be used to print a document from an application using the Gnome Chemistry Utils.
-*/
-	virtual void Print (GnomePrintContext *pc);
-/*!
 @param w: the GtkWidget inside which the Object is displayed.
 
 Used to update the representation of the Object in the widget. This method might be overrided for displayable Object classes
@@ -403,7 +396,7 @@ passed as parameters. Default implementation returns NULL.
 This method is called to build a parent object from its children. The object must already exist.
 @return true in case of success and false if failed.
 */
-	virtual bool Build (list<Object*>& Children) throw (invalid_argument);
+	virtual bool Build (std::list<Object*>& Children) throw (std::invalid_argument);
 
 /*!
 Used to retreive the y coordinate for alignment. The default implementation returns 0.0 and
@@ -470,7 +463,7 @@ Use this function to retrieve the first object linked to the object and initiali
 Links can be used when the relation between the objects is not a parent to child one.
 @return the first object linked to the object or NULL.
 */
-	Object* GetFirstLink (set<Object*>::iterator& i);
+	Object* GetFirstLink (std::set<Object*>::iterator& i);
 
 /*!
 @param i: a C++ std::set<Object*> iterator initialized by Object::GetFirstLink.
@@ -478,7 +471,7 @@ Links can be used when the relation between the objects is not a parent to child
 Use this method to iterate through the list of Object instances linked to the object.
 @return the next object linked to the object or NULL.
 */
-	Object* GetNextLink (set<Object*>::iterator& i);
+	Object* GetNextLink (std::set<Object*>::iterator& i);
 
 /*!
 @param object: the object to unlink.
@@ -500,7 +493,36 @@ directly, but should call Object::OnUnlink instead.
 
 Fills types with all valid ancestor types for the object as defined by rules created with AddRule
 */
-	void GetPossibleAncestorTypes (set<TypeId>& types);
+	void GetPossibleAncestorTypes (std::set<TypeId>& types);
+
+/*!
+@param property: the property id as defined in objprops.h
+@param value: the property value as a string
+
+Used when loading to set properties to various objects. If the method retruns false,
+the property should be set again later. This might happen if an atom does not exists when one 
+of its bonds is loaded.
+@return true if the property could be set, or if the property is not relevant, false otherwise.
+*/
+	virtual bool SetProperty (unsigned property, char const *value);
+
+/*!
+@param property: the property id as defined in objprops.h
+
+Used when saving to get properties from various objects.
+*/
+	virtual std::string GetProperty (unsigned property) const;
+
+/*!
+This method should be called when an object has been fully loaded. The default method doesn't do anything.
+*/
+	virtual void OnLoaded ();
+	
+/*!
+@param dirty should be true if the object needs some update, false otherwise. For a document,
+it  means that the document has been changed.
+*/
+	void SetDirty (bool dirty = true);
 
 /*!
 @param TypeName: the name of the new type.
@@ -511,7 +533,7 @@ can be omitted.
 This method is used to register a new type derived from Object.
 @return the Id of the new type.
 */
-	static TypeId AddType (string TypeName, Object* (*CreateFunc) (), TypeId id = OtherType);
+	static TypeId AddType (std::string TypeName, Object* (*CreateFunc) (), TypeId id = OtherType);
 
 /*!
 @param TypeName: the name of the new type.
@@ -523,21 +545,21 @@ It will also be given a default Id.
 
 @return a pointer to the newly created Object or NULL if the Object could not be created.
 */
-	static Object* CreateObject (const string& TypeName, Object* parent = NULL);
+	static Object* CreateObject (const std::string& TypeName, Object* parent = NULL);
 
 /*!
 @param Name: the name of the Object derived class
 
 @return the TypeId corresponding to Name
 */
-	static TypeId GetTypeId (const string& Name);
+	static TypeId GetTypeId (const std::string& Name);
 
 /*!
 @param Id: the TypeId of the Object derived class
 
 @return the name of the type.
 */
-	static string GetTypeName (TypeId Id);
+	static std::string GetTypeName (TypeId Id);
 
 /*!
 @param Id: the TypeId of the Object derived class
@@ -563,7 +585,7 @@ Adds a rule.
 
 Adds a rule.
 */
-	static void AddRule (const string& type1, RuleId rule, const string& type2);
+	static void AddRule (const std::string& type1, RuleId rule, const std::string& type2);
 
 /*!
 @param type: the TypeId of a class
@@ -571,7 +593,7 @@ Adds a rule.
 
 @return the set of rules correponding to the RuleId value for this class.
 */
-	static  const set<TypeId>& GetRules (TypeId type, RuleId rule);
+	static  const std::set<TypeId>& GetRules (TypeId type, RuleId rule);
 
 /*!
 @param type: the name of a class
@@ -579,7 +601,7 @@ Adds a rule.
 
 @return the set of rules correponding to the RuleId value for this class.
 */
-	static const set<TypeId>& GetRules (const string& type, RuleId rule);
+	static const std::set<TypeId>& GetRules (const std::string& type, RuleId rule);
 
 /*!
 @param Id: the TypeId of a class
@@ -588,21 +610,21 @@ Adds a rule.
 Used to give a label for contextual menus used when the creation of an instance of
 the class seems possible.
 */
-	static void SetCreationLabel (TypeId Id, string Label);
+	static void SetCreationLabel (TypeId Id, std::string Label);
 
 /*!
 @param Id: the TypeId of a class
 
 @return the string defined by SetCreationLabel.
 */
-	static const string& GetCreationLabel (TypeId Id);
+	static const std::string& GetCreationLabel (TypeId Id);
 
 /*!
 @param TypeName: the name of a class
 
 @return the string defined by SetCreationLabel.
 */
-	static const string& GetCreationLabel (const string& TypeName);
+	static const std::string& GetCreationLabel (const std::string& TypeName);
 
 /*!
 @return a new SignalId.
@@ -616,14 +638,20 @@ private:
 	gchar* m_Id;
 	TypeId m_Type;
 	Object *m_Parent;
-	map<string, Object*> m_Children; //string is Id of object, so each object must have an Id
-	set<Object*> m_Links; //objects linked to this but outside of the hierarchy
+	std::map<std::string, Object*> m_Children; //string is Id of object, so each object must have an Id
+	std::set<Object*> m_Links; //objects linked to this but outside of the hierarchy
 
 private:
 /*!
 Set to true while loading to avoid signal propagation.
 */
 	int m_Locked;
+
+/*!\fn GetDirty()
+@return true if the document has changed since it was opened or last saved,
+false otherwise.
+*/
+GCU_RO_PROP (bool, Dirty);
 };
 
 }
