@@ -25,10 +25,6 @@
 #include "config.h"
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnomeprint/gnome-print.h>
-#include <libgnomeprint/gnome-print-job.h>
-#include <libgnomeprintui/gnome-print-dialog.h>
-#include <libgnomeprintui/gnome-print-job-preview.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include "application.h"
@@ -98,48 +94,6 @@ bool gcApplication::OnFileClose ()
 	return true;
 }
 
-void gcApplication::OnFilePrint ()
-{
-	GnomePrintConfig* config = gnome_print_config_default ();
-	GnomePrintContext *pc;
-	GnomePrintJob *gpj = gnome_print_job_new (config);
-	int do_preview = 0, copies = 1, collate = 0;
-	GnomePrintDialog *gpd;
-	gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new (gpj, (const guchar*) "Print test", GNOME_PRINT_DIALOG_COPIES));
-	gtk_window_set_icon_name (GTK_WINDOW (gpd), "gcrystal");
-	gnome_print_dialog_set_copies (gpd, copies, collate);
-	switch (gtk_dialog_run (GTK_DIALOG (gpd))) {
-	case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
-		do_preview = 0;
-		break;
-	case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
-		do_preview = 1;
-		break;
-	case GNOME_PRINT_DIALOG_RESPONSE_CANCEL:
-		gtk_widget_destroy (GTK_WIDGET (gpd));
-		return;
-	}
-	gtk_widget_destroy (GTK_WIDGET (gpd));
-	pc = gnome_print_job_get_context (gpj);
-	gnome_print_beginpage (pc, (const guchar*) "");
-	gdouble width, height;
-	gnome_print_config_get_double (config, (const guchar*) GNOME_PRINT_KEY_PAPER_WIDTH, &width);
-	gnome_print_config_get_double (config, (const guchar*) GNOME_PRINT_KEY_PAPER_HEIGHT, &height);
-	m_pActiveDoc->GetActiveView ()->Print (pc, width, height);
-	gnome_print_showpage (pc);
-	g_object_unref (pc);
-	gnome_print_job_close (gpj);
-	if (do_preview) {
-		GtkWidget *preview = gnome_print_job_preview_new (gpj, (const guchar*)_("Preview"));
-		gtk_window_set_icon_name (GTK_WINDOW (preview), "gcrystal");
-		gtk_widget_show (preview);
-	} else {
-		gnome_print_job_print (gpj);
-	}
-	g_object_unref (gpj);
-	gnome_print_config_unref (config);
-}
-
 void gcApplication::OnSaveAsImage ()
 {
 	if (!m_pActiveDoc)
