@@ -26,6 +26,7 @@
 #define GCHEMTABLE_CURVE_H
 
 #include <gcu/dialog.h>
+#include <gcu/printable.h>
 #include <string>
 #include "gchemtable-app.h"
 #include <goffice/graph/gog-graph.h>
@@ -34,7 +35,7 @@
 
 using namespace gcu;
 
-class GChemTableCurve: public Dialog
+class GChemTableCurve: public Dialog, public Printable
 {
 public:
 	GChemTableCurve (GChemTableApp *App, char const *name);
@@ -43,20 +44,18 @@ public:
 	GChemTableApp *GetApplication () {return dynamic_cast <GChemTableApp *> (m_App);}
 
 	void OnPageSetup ();
-	void OnPrint (bool preview);
 	void OnCopy ();
 	void OnClose ();
 	void OnProperties ();
 
 	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context);
 	void SetGraph (GogGraph *graph);
+	GtkWindow *GetGtkWindow () {return GTK_WINDOW (dialog);}
 
 private:
 	std::string m_Name;
 	GogGraph *m_Graph;
 	GtkWidget *m_GraphWidget, *m_GraphBox;
-	GtkPrintSettings *m_PrintSettings;
-	GtkPageSetup *m_PageSetup;
 };
 
 #endif	// GCHEMTABLE_CURVE_H
